require 'spec_helper'
require 'emcee/agent'
require 'json'

describe Emcee::Agent do
  before do
    @runner = Emcee::Runner.new
    @agent_data = fake_results.first[:data][:agents].first
    @agent = Emcee::Agent.new(@runner, @agent_data)
  end

  describe 'when initializing' do
    before do
      @agent_data = fake_results.first[:data][:agents].first
    end

    it 'allows specifying a runner and a hash of agent data' do
      lambda { Emcee::Agent.new(@runner, @agent_data) }.should_not raise_error(ArgumentError)
    end

    it 'requires a runner and a hash of agent data' do
      lambda { Emcee::Agent.new(@runner) }.should raise_error(ArgumentError)
    end

    it 'makes the runner available' do
      Emcee::Agent.new(@runner, @agent_data).runner.should == @runner
    end

    it 'makes the agent name available' do
      Emcee::Agent.new(@runner, @agent_data).name.should == @agent_data[:agent]
    end

    it 'makes the agent version available' do
      Emcee::Agent.new(@runner, @agent_data).version.should == @agent_data[:version]
    end
  end

  describe 'outputting to JSON format' do
    it 'should produce JSON that can be used to reconstruct a known data structure' do
      @ddl = stub(:actions => ['purge'])
      @agent.stubs(:ddl).returns(@ddl)
      @agent.action_names.each {|name| @agent.stubs(:action_interface).with(name).returns(fake_interfaces[name])}
      [Emcee::MCNode.new('foo'), Emcee::MCNode.new('bar')].each {|n| @agent.add_node(n)}
      JSON.parse(@agent.to_json).should == { 
        'name'        => 'discovery',
        'version'     => 'unknown',
        'description' => 'unknown',
        'nodes'       => ['foo', 'bar'],
        'actions'     => [{ 'name'        => 'purge',
                            'description' => 'Purge a package',
                            'input'       => { 
                              'package' => { 
                                'maxlength'   => 90, 
                                'type'        => 'string', 
                                'description' => 'Package to Purge', 
                                'prompt'      => 'Package Name', 
                                'optional'    => false, 
                                'validation'  => '.'
                              }
                            }
                          }]
      }
    end
  end

  describe 'matching other agents' do
    describe 'when the other agent is an agent object' do
      it 'does not match when the agents have differing versions' do
        @other = Emcee::Agent.new(@runner, :agent => @agent.name, :version => 'random version string')
        @agent.matches?(@other).should be_false
      end

      it 'does not match when the agents have differing names' do
        @other = Emcee::Agent.new(@runner, :agent => 'random agent name', :version => @agent.version)
        @agent.matches?(@other).should be_false
      end

      it 'matches when the agents have identical versions and names' do
        @other = Emcee::Agent.new(@runner, :agent => @agent.name, :version => @agent.version)
        @agent.matches?(@other).should be_true
      end
    end

    describe 'when the other agent is an agent data hash' do
      describe 'and both agent name and version are specified' do
        it 'does not match when the agents have differing versions' do
          @agent.matches?(:agent => @agent.name, :version => 'random version string').should be_false
        end

        it 'does not match when the agents have differing names' do
          @agent.matches?(:agent => 'random agent name', :version => @agent.version).should be_false
        end

        it 'matches when the agents have identical versions and names' do
          @agent.matches?(:agent => @agent.name, :version => @agent.version).should be_true
        end
      end


      describe 'and only agent name is specified' do
        it 'matches agents with the same name' do
          @agent.matches?(:agent => @agent.name).should be_true
        end

        it 'does not match agents with differing names' do
          @agent.matches?(:agent => 'someothername').should be_false
        end
      end

      describe 'and only agent version is specified' do
        it 'matches agents with the same version' do
          @agent.matches?(:version => @agent.version).should be_true
        end

        it 'does not match agents with differing versions' do
          @agent.matches?(:version => '0.0').should be_false
        end
      end

      describe 'and neither agent name nor agent version are specified' do
        it 'should return false' do
          @agent.matches?(:foo => 'bar', :bar => 'baz').should be_false
        end
      end
    end

    describe 'when other is neither an agent object nor an agent data hash' do
      it 'should return false' do
        @agent.matches?("a string").should be_false
      end
    end

    describe 'finding the agent DDL' do
      it 'returns the DDL for this agent from the runner' do
        @ddl = {}
        @runner.stubs(:ddl).with(@agent.name).returns(@ddl)
        @agent.ddl.should == @ddl
      end
    end

    describe 'finding the list of names of available actions' do
      it 'returns the empty list when the DDL is not found' do
        @runner.stubs(:ddl).with(@agent.name).returns(nil)
        @agent.action_names.should == []
      end

      it 'returns the actions listed in the agent DDL' do
        @actions = [ 'new', 'edit', 'delete' ]
        @ddl = stub(:actions => @actions)
        @runner.stubs(:ddl).with(@agent.name).returns(@ddl)
        @agent.action_names.should == @actions
      end
    end
  end

  describe 'adding nodes' do
    before do
      @node = Emcee::MCNode.new('node')
    end

    it 'requires a node' do
      lambda { @agent.add_node }.should raise_error(ArgumentError)
    end

    it 'makes the node available in the agent node list' do
      @agent.add_node(@node)
      @agent.nodes.should include(@node)
    end

    it 'does not add a node if that node is already in the agent node list' do
      @node.add_agent(@agent)
      @other = Emcee::Agent.new(@runner, :agent => @agent.name, :version => @agent.version)
      @node.add_agent(@other)
      @node.agents.size.should == 1
    end

    it 'does not add a node if that node is already in the agent node list' do
      @agent.add_node(@node)
      @agent.add_node(Emcee::MCNode.new(@node.name))
      @agent.nodes.size.should == 1
    end
  end

  describe 'when finding actions by name' do
    before do
      @ddl = stub(:actions => [ 'status', 'yum_checkupdates' ])
      @agent.stubs(:ddl).returns(@ddl)
      @fake_interfaces = fake_interfaces
      @agent.action_names.each {|name| @agent.stubs(:action_interface).with(name).returns(@fake_interfaces[name])}
    end

    it 'should return any action that matches the passed string' do
      action = @agent.action_by_name('status')
      action.class.should == Emcee::Action
      action.name.should == 'status'
    end

    it 'should return false if there is no match' do
      @agent.action_by_name('missing').should be_false
    end
  end
end

