require 'spec_helper'

require 'emcee'

describe Emcee do
  before do
    Emcee::Runner.any_instance.stubs(:results).returns(fake_results)
    @emcee = Emcee.new

    # the fake data doesn't have dashnode 6
    @node_names = ((1..5).to_a + (7..10).to_a).map {|n| "dashnode#{n}"}
  end

  describe '.logger' do
    it 'returns a Logger instance' do
      Emcee.logger.class.should == Logger
    end
  end

  describe 'when initializing' do
    after :each do
      Emcee::Cache.set_strategy(nil)
    end

    it 'sets a filter accessor when passed in' do
      emcee = Emcee.new(:filter => 'myfilter')
      emcee.filter.should == 'myfilter'
    end

    it 'sets a cache accessor when cache strategy is passed in' do
      emcee = Emcee.new(:cache => :none)
      emcee.cache_strategy.should == :none
    end

    it 'notifies Emcee::Cache of the desired cache strategy' do
      emcee = Emcee.new(:cache => :file)
      emcee.cache_strategy.should == :file
    end
  end

  describe 'returning a node list' do
    it 'returns the list of node instances based upon an agent inventory' do
      @emcee.nodes.map(&:name).should =~ @node_names
    end
  end

  describe 'returning an agent list' do
    it 'returns the list of agent instances based upon an agent inventory' do
      @emcee.agents.collect {|a| "#{a.name}:#{a.version}" }.should =~ [
        "discovery:unknown",
        "nrpe:1.2",
        "nrpe:1.3",
        "package:1.3",
        "registration:unknown",
        "rpcutil:1.0",
        "service:1.2",
        "test:version"
      ]
    end
  end

  describe 'finding a specific node by node name' do
    it 'returns nil if no such node can be found' do
      @emcee.node('nonexistant').should be_nil
    end

    it 'returns the node instance if such a node can be found' do
      @emcee.node('dashnode3').name.should == 'dashnode3'
    end
  end

  describe 'finding a specific agent by agent data' do
    it 'returns nil if no such agent can be found' do
      @emcee.agent(:agent => 'foo', :version => 'bar').should be_nil
    end

    it 'returns the node instance if such a node can be found' do
      result = @emcee.agent(:agent => 'package', :version => '1.3')
      result.name.should == 'package'
      result.version.should == '1.3'
    end
  end

  describe 'finding agents by agent name' do
    it 'returns the empty list when there are no agents' do
      @emcee.stubs(:agents).returns([])
      @emcee.agents_by_name('foo').should == []
    end

    it 'returns the empty list when no agents match' do
      @emcee.agents_by_name('nomatch').should == []
    end

    it 'returns the agents with the same name' do
      @emcee.agents_by_name('nrpe').size.should == 2
    end

    it 'returns only those agents with the same name' do
      @emcee.agents_by_name('nrpe').each do |agent|
        agent.name.should == 'nrpe'
      end
    end
  end

  describe 'finding filtered nodes' do
    it 'returns the list of filtered nodes from the runner' do
      filtered_nodes = ['1', '2', '3']
      Emcee::Runner.any_instance.stubs(:filtered_nodes).returns(filtered_nodes)
      @emcee.filtered_nodes.should == filtered_nodes
    end
  end
end
