require 'spec_helper'
require 'resource_management'
require 'emcee'
require 'emcee/result'

describe ResourceManagement do

  describe '#string_to_nodes_regex' do

    it 'should be the identity on strings that are not special' do
      ResourceManagement.string_to_nodes_regex('foobar').should == 'foobar'
      ResourceManagement.string_to_nodes_regex('foo.bar').should == 'foo.bar'
      ResourceManagement.string_to_nodes_regex('/foo.bar').should == '/foo.bar'
      ResourceManagement.string_to_nodes_regex('foo.bar/').should == 'foo.bar/'
    end

    it 'should disallow regular expressions' do
      ResourceManagement.string_to_nodes_regex('/foobar/').should == 'foobar'
      ResourceManagement.string_to_nodes_regex('/foo.bar/').should == 'foo.bar'
      ResourceManagement.string_to_nodes_regex('/fo/o.bar/').should == 'fo/o.bar'
    end

    it 'should understand the * and ? globbing characters' do
      ResourceManagement.string_to_nodes_regex('foo*bar').should == '/^foo.*bar$/'
      ResourceManagement.string_to_nodes_regex('foo?bar').should == '/^foo.bar$/'
      ResourceManagement.string_to_nodes_regex('foo?bar*').should == '/^foo.bar.*$/'
      ResourceManagement.string_to_nodes_regex('/foo*bar/').should == '/^foo.*bar$/'
      ResourceManagement.string_to_nodes_regex('/foo?bar/').should == '/^foo.bar$/'
      ResourceManagement.string_to_nodes_regex('foo.*bar/').should == '/^foo\..*bar/$/'
    end

  end

  describe '#clone' do

    before do
      @emcee = Emcee.new
      Emcee.stubs(:new).returns(@emcee)
    end

    it 'should call Emcee.create_resource with the described resource' do
      resource = {
        'type'       => 'foo',
        'title'      => 'bar',
        'parameters' => { :ensure => :present }
      }
      @emcee.expects(:create_resource).with(resource).returns([])
      ResourceManagement.clone(['node1'], resource)
    end

    describe "when the resource is of type user" do
      it "should call Emcee.create_resource with an :avoid_conflict value of 'uid'" do
        resource = {
          'type'       => 'user',
          'title'      => 'foobar',
          'parameters' => { :ensure => :present }
        }
        @emcee.expects(:create_resource).with(resource.dup.merge({'avoid_conflict' => 'uid'})).returns([])
        ResourceManagement.clone(['node1'], resource)
      end

      it 'should create a user and secondary groups' do
        user_resource = {
          'type'       => 'user',
          'title'      => 'foobar',
          'parameters' => {
            :ensure => :present,
            :groups => ['bork']
          }
        }
        group_resource = {
          'type'       => 'group',
          'title'      => 'bork',
          'parameters' => { :ensure => :present }
        }
        target_nodes = ['node1']
        @emcee.
          expects(:resources).never
        expected_group = group_resource.dup.merge({'avoid_conflict' => 'gid' })
        @emcee.
          expects(:create_resource).
          with(expected_group).
          returns([])
        expected_user = user_resource.dup.merge({ 'avoid_conflict' => 'uid' })
        @emcee.
          expects(:create_resource).
          with(expected_user).
          returns([])
        ResourceManagement.clone(target_nodes, user_resource)
      end

      def make_return_data(title, attributes)
        {
          :successes => [{
            "variations" => [{
              :nodes => ['node1'],
              :attributes => attributes
            }],
            "title"      => title,
            "nodeCount"  => 1,
            "properties" => attributes.keys }],
          :failures  => [],
          :errors    => [] }
      end

      it "should create the user's primary group" do
        # param keys are strings since they come from browser front end via json
        user_resource = {
          'type'       => 'user',
          'title'      => 'foobar',
          'parameters' => {
            'ensure' => 'present',
            'groups' => ['bork'],
            'group'  => 'mork',
          }
        }
        group_resource = {
          'type'       => 'group',
          'title'      => 'bork',
          'parameters' => { :ensure => :present }
        }
        gid_group = {
          'type'       => 'group',
          'title'      => 'mork',
          'parameters' => { :ensure => :present }
        }
        target_nodes = ['node1']
        expected_group = group_resource.dup.merge({ 'avoid_conflict' => 'gid' })
        ResourceManagement.
          expects(:shallow_clone).
          with(['node1'], expected_group).
          returns(make_return_data('bork', group_resource["parameters"]))
        new_gid = 123
        group_attributes = gid_group["parameters"].dup
        group_attributes[:gid] = new_gid
        ResourceManagement.
          expects(:shallow_clone).
          with(['node1'], gid_group.dup.merge({ 'avoid_conflict' => 'gid' }), false).
          returns(make_return_data('mark', group_attributes))
        user_attributes = {}
        user_attributes[:gid] = new_gid
        user_resource["parameters"].each do |key, value|
          user_attributes[key.to_sym] = value unless key == 'group'
        end
        expected_user = user_resource.dup.merge({ 'avoid_conflict' => 'uid' })
        expected_user['parameters'] = user_attributes
        ResourceManagement.
          expects(:shallow_clone).
          with(['node1'], expected_user).
          returns(make_return_data('user', user_attributes))
        ResourceManagement.clone(target_nodes, user_resource)
      end
    end

    describe "when the resource is of type group" do
      it "should call Emcee.create_resource with an :avoid_conflict value of 'gid'" do
        resource = {
          'type'       => 'group',
          'title'      => 'foobar',
          'parameters' => { :ensure => :present }
        }
        @emcee.expects(:create_resource).with(resource.merge({'avoid_conflict' => 'gid'})).returns([])
        ResourceManagement.clone(['node1'], resource)
      end
    end
  end

  describe '#enforce_parameter_whitelist!' do

    it 'should reduce the passed parameters of a user to a minimal whitelist' do
      resource = {
        'type'       => 'user',
        'title'      => 'foobar',
        'parameters' => {
          :ensure => 'present',
          :bad_param => "This is an invalid parameter"
        }
      }
      resource_without_param = {
        'type'       => 'user',
        'title'      => 'foobar',
        'parameters' => {
          :ensure => 'present'
        }
      }
      ResourceManagement.enforce_parameter_whitelist!(resource)
      resource.should == resource_without_param
    end

  end

  describe '#consolidate_successes' do

    it 'should return an empty array if there are no successes' do
      ResourceManagement.send(:consolidate_successes, []).should == []
    end

    it 'should remove duplicate variations while keeping track of the nodes on which those variations occur' do
      a =  {
        'title'      => 'bork',
        'nodeCount'  => 2,
        'properties' => ['foo', 'bar', 'baz'],
      }
      b = a.dup
      c = a.dup
      a['variations'] = [ { :attributes => { 'foo' => 1, 'bar' => -1, 'baz' => 0.1 },
                            :nodes      => [ 'node1', 'node2' ] } ]
      b['variations'] = [ { :attributes => { 'foo' => 1, 'bar' => -1, 'baz' => 0.1 },
                            :nodes      => [ 'node3', 'node4' ] },
                          { :attributes => { 'foo' => 2, 'bar' => -2, 'baz' => 0.2 },
                            :nodes      => [ 'node5', 'node6' ] } ]
      c['variations'] = [ { :attributes => { 'foo' => 1, 'bar' => -1, 'baz' => 0.1 },
                            :nodes      => [ 'node7', 'node8' ] },
                          { :attributes => { 'foo' => 2, 'bar' => -2, 'baz' => 0.2 },
                            :nodes      => [ 'node9', 'node10' ] } ]
      successes = [a, b, c]
      expected = a.dup
      expected['variations'] = [ { :attributes => { 'foo' => 2, 'bar' => -2, 'baz' => 0.2 },
                                   :nodes      => [ 'node5', 'node6', 'node9', 'node10' ] },
                                 { :attributes => { 'foo' => 1, 'bar' => -1, 'baz' => 0.1 },
                                   :nodes      => [ 'node1', 'node2', 'node3', 'node4', 'node7', 'node8' ] }]

      ResourceManagement.send(:consolidate_successes, successes).should == [ expected ]
    end

  end

  describe '#bare_group_resource' do

    it 'should return a properly formed minimal group resource' do
      ResourceManagement.send(:bare_group_resource, 'foo').should == {
        'type'           => 'group',
        'title'          => 'foo',
        'avoid_conflict' => 'gid',
        'parameters'     => { :ensure => :present }
      }
    end

  end

  describe '#frontend_encode_groups!' do

    it 'removes the :gid parameter from each group' do
      before = [{ 'type'       => 'group',
                  'title'      => 'foo',
                  'parameters' => { :gid => 1000, :ensure => :present } }]
      after = [{ 'type'       => 'group',
                 'title'      => 'foo',
                 'parameters' => { :ensure => :present } }]
      ResourceManagement.send(:frontend_encode_groups!, before)
      before.should == after
    end

  end

  describe '#frontend_encode_users!' do

    it "removes the :uid parameter and replaces the :gid parameter with :group on each user and doesn't change the input groups" do
      before_users = [{ 'type'       => 'user',
                        'title'      => 'foo',
                        'parameters' => {
                          :uid      => 1000,
                          :gid      => 1001,
                          :home     => '/home/foo',
                          :groups   => ['bar', 'baz'] } }]
      before_groups = [{ 'type'       => 'group',
                         'title'      => 'bork',
                         'parameters' => { :gid => 1001 } }]
      after_users = [{ 'type'       => 'user',
                       'title'      => 'foo',
                       'parameters' => {
                         :group    => 'bork',
                         :home     => '/home/foo',
                         :groups   => ['bar', 'baz'] } }]
      after_groups = before_groups.dup
      ResourceManagement.send(:frontend_encode_users!, before_users, before_groups)
      before_users.should == after_users
      before_groups.should == after_groups
    end

    it "removes the :uid parameter and doesn't change the input groups or add the :group parameter if there is no :gid parameter" do
      before_users = [{ 'type'       => 'user',
                        'title'      => 'foo',
                        'parameters' => {
                          :password => 'abc',
                          :uid      => 1000,
                          :home     => '/home/foo',
                          :groups   => ['bar', 'baz'] } }]
      before_groups = [{ 'type'       => 'group',
                         'title'      => 'bork',
                         'parameters' => { :gid => 1001 } }]
      after_users = [{ 'type'       => 'user',
                       'title'      => 'foo',
                       'parameters' => {
                         :password => 'abc',
                         :home     => '/home/foo',
                         :groups   => ['bar', 'baz'] } }]
      after_groups = before_groups.dup
      ResourceManagement.send(:frontend_encode_users!, before_users, before_groups)
      before_users.should == after_users
      before_groups.should == after_groups
    end

  end

end
