require 'spec_helper'

describe BaselineDifference do
  describe "validation" do
    it "should accept neither accepted or decided_at" do
      BaselineDifference.new.should be_valid
    end

    it "should accept both accepted and decided_at" do
      BaselineDifference.new(:accepted => true, :decided_at => Time.now).should be_valid
    end

    it "should reject only accepted" do
      BaselineDifference.new(:accepted => true).should_not be_valid
    end

    it "should reject only decided_at" do
      BaselineDifference.new(:decided_at => Time.now).should_not be_valid
    end
  end

  describe "#apply_decision_to" do
    it "should apply an accepted change to the baseline" do
      baseline_report = Report.create_from_yaml(File.read(File.join(File.dirname(__FILE__), '../fixtures/reports/baseline.yaml')))
      report = Report.create_from_yaml(File.read(File.join(File.dirname(__FILE__), '../fixtures/reports/inspect.yaml')))

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Summarizer.build_baseline_summary(report.id, node_baseline)

      differences = node_baseline.differences

      change = differences.find_by_resource_name('File[/etc/hosts]')
      change.accept_change

      change.apply_decision_to(node_baseline.baseline)

      new_resource = node_baseline.baseline.resources.find_by_resource_type_and_title('File', '/etc/hosts')

      new_resource.properties.map {|prop| [prop.name, prop.value]}.should =~ [
        ['ensure',  :file],
        ['content', '{md5}6cd31951036020aee7ff2d27e3953063'],
        ['owner',   '0'],
        ['group',   '0'],
        ['mode',    '644'],
        ['type',    'file'],
        ['ctime',   Time.parse('2011-06-15 09:53:48 -07:00')],
        ['mtime',   Time.parse('2011-06-15 09:53:48 -07:00')]
      ]
    end

    it "should remove accepted resources which were in the previous baseline and not in the report" do
      baseline_report = Report.create_from_yaml(File.read(File.join(File.dirname(__FILE__), '../fixtures/reports/baseline.yaml')))
      report = Report.create_from_yaml(File.read(File.join(File.dirname(__FILE__), '../fixtures/reports/inspect.yaml')))

      report.resource_statuses.find_by_resource_type_and_title('File', '/etc/hosts').destroy
      report.reload

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Summarizer.build_baseline_summary(report.id, node_baseline)

      differences = node_baseline.differences

      change = differences.find_by_resource_name('File[/etc/hosts]')
      change.accept_change

      change.apply_decision_to(node_baseline.baseline)

      node_baseline.baseline.resources.find_by_resource_type_and_title('File', '/etc/hosts').should_not be_present
    end

    it "should add accepted resources which were in the report and not in the previous baseline" do
      baseline_report = Report.create_from_yaml(File.read(File.join(File.dirname(__FILE__), '../fixtures/reports/baseline.yaml')))
      report = Report.create_from_yaml(File.read(File.join(File.dirname(__FILE__), '../fixtures/reports/inspect.yaml')))

      baseline_report.resource_statuses.find_by_resource_type_and_title('File', '/etc/hosts').destroy
      baseline_report.reload

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Summarizer.build_baseline_summary(report.id, node_baseline)

      differences = node_baseline.differences

      change = differences.find_by_resource_name('File[/etc/hosts]')
      change.accept_change

      change.apply_decision_to(node_baseline.baseline)

      node_baseline.baseline.resources.find_by_resource_type_and_title('File', '/etc/hosts').should be_present
    end

    it "should work if you accept two sequential 'unexists' inspected values" do
      # This test is very intimate with the workings of the baseline system.
      # If it breaks when you make logic changes, that is probably the fault
      # of the test for being so close.  More abstract testing is more costly
      # than I can manage right now, sorry. --daniel 2011-08-09
      start = (Time.now - 2.days)

      reports =
        [Factory(:inspect_report, :host => 'bomf.local', :time => start),
         Factory(:inspect_report, :host => 'bomf.local', :time => start + 1.day),
         Factory(:inspect_report, :host => 'bomf.local', :time => start + 2.days)]

      Factory(:successful_resource, :title => 'good', :report => reports[0])

      # Get the node-baseline pairs in place, and diff them against the report.
      nb = [NodeBaseline.create_from_report! reports[0], reports[0].time]
      Summarizer.build_baseline_summary(reports[0].id, nb[0])

      # Create the two subsequent node-baseline pairs.
      nb << nb.last.create_for_next_day
      nb << nb.last.create_for_next_day

      # Difference all those reports against their baseline.
      (0..2).each do |n|
        Summarizer.build_baseline_summary(reports[n].id, nb[n])
      end

      # Accept both differences, right now.
      BaselineDifference.all.map(&:accept_change)

      # ...and, finally, our meat:
      final = nb.last.create_for_next_day

      # ...but, really, our test is "we didn't raise in the last method"
      final.baseline.id.should_not == nb.last.id
      final.baseline.resources.count.should == 0
    end
  end

  describe "#accept_change" do
    def node_baselines(*values)
      start = Time.now - values.length.days
      reports = []

      values.each_with_index do |value, i|
        reports << Factory(:inspect_report,
                           :host => 'accept.changes',
                           :time => start + i.days)

        Factory(:resource_event,
                :resource_status => Factory(:resource_status,
                                            :report => reports.last,
                                            :title  => '/example'),
                :property        => 'example',
                :previous_value  => value)
      end

      # Generate the NodeBaseline records...
      nb = [NodeBaseline.create_from_report!(reports[0], reports[0].time)]
      (reports.length - 1).times { nb << nb.last.create_for_next_day }

      # ...and the differences.
      Summarizer.stubs(:delay).returns(Summarizer)
      Summarizer.queue_diff_summarization

      nb
    end

    def to_accepted(nb)
      nb.map do |this|
        diff = this.differences
        diff.empty? ? nil : diff.first.accepted
      end
    end

    {
      :accept     => proc {|x| x.accept_change },
      :reject     => proc {|x| x.reject_change },
      :accept_all => proc {|x| BaselineDifference.accept_all_changes([x]) },
      :reject_all => proc {|x| BaselineDifference.reject_all_changes([x]) }
    }.each do |which, how|
      want = which.to_s.include? 'accept'

      it "should #{which} earlier identical changes" do
        nb = node_baselines('original', 'changed', 'changed')
        how.call nb.last.differences.first
        to_accepted(nb).should == [nil, want, want]
      end

      it "should not #{which} earlier different changes" do
        nb = node_baselines('original', 'changed', 'again')
        how.call nb.last.differences.first
        to_accepted(nb).should == [nil, nil, want]
      end

      it "should not #{which} identical changes over a discontinuity out of compliance" do
        nb = node_baselines('original', 'change', 'change', 'again', 'change', 'change')
        how.call nb.last.differences.first
        to_accepted(nb).should == [nil, nil, nil, nil, want, want]
      end

      it "should not #{which} identical changes over a discontinuity in compliance" do
        nb = node_baselines('orig', 'change', 'change', 'orig', 'change', 'change')
        how.call nb.last.differences.first
        to_accepted(nb).should == [nil, nil, nil, nil, want, want]
      end

      it "should not #{which} future identical change" do
        nb = node_baselines('original', 'changed', 'changed', 'changed')
        how.call nb[2].differences.first
        to_accepted(nb).should == [nil, want, want, nil]
      end

      it "should not #{which} over an existing decision that agrees" do
        nb = node_baselines('original', 'changed', 'changed', 'changed')
        diff = nb[2].differences.first
        diff[:accepted]   = want
        diff[:decided_at] = Time.now
        diff.save

        how.call nb.last.differences.first
        to_accepted(nb).should == [nil, nil, want, want]
      end

      it "should not #{which} over an existing decision that disagrees" do
        nb = node_baselines('original', 'changed', 'changed', 'changed')
        diff = nb[2].differences.first
        diff[:accepted]   = !want
        diff[:decided_at] = Time.now
        diff.save

        how.call nb.last.differences.first
        to_accepted(nb).should == [nil, nil, !want, want]
      end
    end
  end
end
